/*
** pork_acct.h - account management.
** Copyright (C) 2002 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#ifndef __PORK_ACCT_H
#define __PORK_ACCT_H

struct aim_rights {
	u_int32_t max_buddies;
	u_int32_t max_watchers;
	u_int32_t max_permit;
	u_int32_t max_deny;
	u_int32_t max_away_len;
	u_int32_t max_profile_len;
};

struct aim_priv {
	u_int32_t ref_count;

	char *username;
	char *passwd;
	char *profile;
	char *away_msg;

	char *proxy;
	char *proxy_user;
	char *proxy_passwd;
	char *server;

	u_int32_t warn_level;
	u_int32_t idle_time;
	u_int32_t flags;
	u_int32_t connected:1;
	u_int32_t marked_idle:1;
	u_int32_t whois_hack:1;

	time_t last_update;
	time_t last_input;

	aim_session_t aim_session;
	aim_conn_t *bos_conn;
	struct aim_rights rights;

	struct buddy_pref *buddy_pref;
	struct blist *blist;
};

struct aim_acct {
	char *username;
	struct aim_priv *priv;
	aim_session_t *session;
};

struct aim_acct *pork_acct_add(char *user);
int pork_acct_del(char *user);
void pork_acct_del_all(void);
inline dlist_t *pork_acct_find(char *user);
inline struct aim_priv *pork_acct_get_priv(char *user);
void pork_acct_update(void *data, void *nothing __notused);
void acct_update_blist_format(void *data, void *nothing __notused);

struct aim_priv *pork_priv_init(const char *user);
void pork_priv_free(struct aim_priv *priv);

#endif
