/*
** pork_perl.c - Perl scripting support
** Copyright (C) 2002 Ryan McCabe <ryan@numb.org>
**
** The init, execute, and destroy functions were pretty much copied
** from the gaim perl.c source file.
** Copyrights there apply here.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <sys/types.h>

#ifdef USE_PERL

#include <stdio.h>
#include <stdlib.h>
#include <ncurses.h>
#include <sys/mman.h>

#undef PACKAGE
#undef instr
#define HAS_BOOL

#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>

#include <libfaim/aim.h>
#include <libfaim/faimconfig.h>

#include <pork.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_buddy.h>
#include <pork_set.h>
#include <pork_swindow.h>
#include <pork_imwindow.h>
#include <pork_acct.h>
#include <pork_input.h>
#include <pork_bind.h>
#include <pork_screen.h>
#include <pork_screen_cmd.h>
#include <pork_aim.h>
#include <pork_status.h>
#include <pork_screen.h>
#include <pork_timer.h>
#include <pork_command.h>
#include <pork_slist.h>
#include <pork_buddy_list.h>
#include <pork_conf.h>
#include <pork_perl.h>
#include <pork_alias.h>
#include <pork_perl_defs.h>

#ifdef OLD_PERL
extern void boot_DynaLoader _((CV * cv));
#define init_args void
#else
extern void boot_DynaLoader(pTHX_ CV* cv);
#define init_args pTHX
#endif

static void xs_init(init_args);

extern struct screen screen;

static PerlInterpreter *perl_inter;

XS(PORK_err_msg) {
	int notused;
	char *msg;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	msg = SvPV(ST(0), notused);
	if (msg == NULL)
		XSRETURN_IV(-1);

	screen_err_msg("%s", msg);
	XSRETURN_IV(0);
}

XS(PORK_status_msg) {
	int notused;
	char *msg;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	msg = SvPV(ST(0), notused);
	if (msg == NULL)
		XSRETURN_IV(-1);

	screen_status_msg("%s", msg);
	XSRETURN_IV(0);
}

XS(PORK_echo) {
	int notused;
	char *msg;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	msg = SvPV(ST(0), notused);
	screen_win_msg(cur_window(), 0, 0, 1, "%s", msg);
	XSRETURN_IV(0);
}

XS(PORK_quit) {
	int exit_val = 0;
	dXSARGS;

	(void) cv;
	
	if (items > 1)
		XSRETURN_IV(-1);

	if (items >= 1)
		exit_val = SvIV(ST(0));

	pork_exit(exit_val, NULL);
	XSRETURN_IV(0);
}

XS(PORK_refresh) {
	dXSARGS;

	(void) cv;
	(void) items;
	
	screen_refresh();
	XSRETURN(0);
}

XS(PORK_save) {
	dXSARGS;

	(void) cv;
	(void) items;
	
	XSRETURN_IV(save_global_config());
}

XS(PORK_bind) {
	char *key_str;
	int key;
	char *command;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 2)
		XSRETURN_IV(-1);

	key_str = SvPV(ST(0), notused);
	command = SvPV(ST(1), notused);

	if (key_str == NULL || command == NULL)
		XSRETURN_IV(-1);

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_IV(-1);

	bind_add(&screen.binds.main, key, command);
	XSRETURN_IV(0);
}

XS(PORK_unbind) {
	char *key_str;
	int key;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	key_str = SvPV(ST(0), notused);
	if (key_str == NULL)
		XSRETURN_IV(-1);

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_IV(-1);

	key = bind_remove(&screen.binds.main, key);
	XSRETURN_IV(0);
}

XS(PORK_bind_get) {
	char *key_str;
	int key;
	int notused;
	struct binding *binding;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_UNDEF;

	key_str = SvPV(ST(0), notused);
	if (key_str == NULL)
		XSRETURN_UNDEF;

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_UNDEF;

	binding = bind_find(&screen.binds.main, key);
	if (binding == NULL)
		XSRETURN_UNDEF;

	XSRETURN_PV(binding->binding);
}

XS(PORK_blist_bind) {
	char *key_str;
	int key;
	char *command;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 2)
		XSRETURN_IV(-1);

	key_str = SvPV(ST(0), notused);
	command = SvPV(ST(1), notused);

	if (key_str == NULL || command == NULL)
		XSRETURN_IV(-1);

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_IV(-1);

	bind_add(&screen.binds.blist, key, command);
	XSRETURN_IV(0);
}

XS(PORK_blist_unbind) {
	char *key_str;
	int key;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	key_str = SvPV(ST(0), notused);
	if (key_str == NULL)
		XSRETURN_IV(-1);

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_IV(-1);

	key = bind_remove(&screen.binds.blist, key);
	XSRETURN_IV(0);
}

XS(PORK_blist_bind_get) {
	char *key_str;
	int key;
	int notused;
	struct binding *binding;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_UNDEF;

	key_str = SvPV(ST(0), notused);
	if (key_str == NULL)
		XSRETURN_UNDEF;

	key = bind_get_keycode(key_str);
	if (key == -1)
		XSRETURN_UNDEF;

	binding = bind_find(&screen.binds.blist, key);
	if (binding == NULL)
		XSRETURN_UNDEF;

	XSRETURN_PV(binding->binding);
}

XS(PORK_alias) {
	char *alias;
	char *cmd;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 2)
		XSRETURN_IV(-1);

	alias = SvPV(ST(0), notused);
	cmd = SvPV(ST(1), notused);

	if (alias == NULL || cmd == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(alias_add(&screen.alias_hash, alias, cmd));
}

XS(PORK_alias_get) {
	char *str;
	int notused;
	struct alias *alias;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_UNDEF;

	str = SvPV(ST(0), notused);
	if (str == NULL)
		XSRETURN_UNDEF;

	alias = alias_find(&screen.alias_hash, str);
	if (alias == NULL)
		XSRETURN_UNDEF;

	XSRETURN_PV(alias->str);
}

XS(PORK_unalias) {
	char *alias;
	int notused;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	alias = SvPV(ST(0), notused);
	if (alias == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(alias_remove(&screen.alias_hash, alias));
}

XS(PORK_set_opt) {
	char *var;
	char *val;
	int unused;
	int opt;
	dXSARGS;

	(void) cv;

	if (items != 2)
		XSRETURN_IV(-1);

	var = SvPV(ST(0), unused);
	val = SvPV(ST(1), unused);

	if (var == NULL || var == NULL)
		XSRETURN_IV(-1);

	opt = opt_find(var);
	if (opt == -1)
		XSRETURN_IV(-1);

	XSRETURN_IV(opt_set(opt, val));
}

XS(PORK_get_opt) {
	char *val;
	int unused;
	char buf[2048];

	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_UNDEF;

	val = SvPV(ST(0), unused);
	if (val == NULL)
		XSRETURN_UNDEF;

	if (opt_get_val(val, buf, sizeof(buf)) == -1)
		XSRETURN_UNDEF;

	XSRETURN_PV(buf);
}

/*
** All 'aim_*' commands take a screen name as their
** first argument.
*/

XS(PORK_aim_set_profile) {
	int notused;
	char *profile;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	profile = SvPV(ST(0), notused);
	if (profile == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_set_profile(priv, profile));
}

XS(PORK_aim_get_profile) {
	int notused;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 0)
		XSRETURN_UNDEF;

	if (items == 1) {
		char *username = SvPV(ST(0), notused);
		if (username == NULL)
			XSRETURN_UNDEF;

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_UNDEF;
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	if (priv->profile == NULL)
		XSRETURN_UNDEF;

	XSRETURN_PV(priv->profile);
}

XS(PORK_aim_send_im) {
	int notused;
	char *dest;
	char *msg;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 2 && items != 3)
		XSRETURN_IV(-1);

	dest = SvPV(ST(0), notused);
	msg = SvPV(ST(1), notused);

	if (dest == NULL || msg == NULL)
		XSRETURN_IV(-1);

	if (items == 3) {
		char *username = SvPV(ST(2), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	return (XSRETURN_IV(aim_send_msg(priv, dest, msg)));
}

XS(PORK_aim_get_buddy_profile) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_whois(priv, target));
}

XS(PORK_aim_get_buddy_away) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_get_away_msg(priv, target));
}

XS(PORK_aim_permdeny_mode) {
	int notused;
	int permdeny_mode;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	permdeny_mode = SvIV(ST(0));

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_permdeny_mode(priv, permdeny_mode));
}

XS(PORK_aim_report_idle) {
	int notused;
	int report_idle;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	report_idle = SvIV(ST(0));

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_report_idle(priv, report_idle));
}

XS(PORK_aim_search) {
	int notused;
	char *target;
	struct aim_priv *priv;
	int ret;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	ret = aim_usersearch_address(&priv->aim_session, priv->bos_conn, target);
	XSRETURN_IV(ret);
}

XS(PORK_aim_set_away) {
	int notused;
	char *msg;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	msg = SvPV(ST(0), notused);
	if (msg == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_set_away(priv, msg));
}

XS(PORK_aim_set_idle) {
	int notused;
	int seconds;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	seconds = SvIV(ST(0));

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(aim_set_idle(priv, seconds));
}

XS(PORK_aim_warn) {
	int notused;
	char *target;
	int anon;
	struct aim_priv *priv;
	int ret;
	dXSARGS;

	(void) cv;

	if (items != 2 && items != 3)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	anon = SvIV(ST(1));

	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 3) {
		char *username = SvPV(ST(2), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	if (anon != 0)
		ret = aim_warn(priv, target);
	else
		ret = aim_warn_anon(priv, target);

	XSRETURN_IV(ret);
}

/*
** Wrappers for non-aim /buddy commands.
*/

XS(PORK_buddy_add) {
	int notused;
	char *target;
	char *group;
	struct aim_priv *priv;
	struct bgroup *bgroup;
	dXSARGS;

	(void) cv;

	if (items != 2 && items != 3)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	group = SvPV(ST(1), notused);

	if (target == NULL || group == NULL)
		XSRETURN_IV(-1);

	if (items == 3) {
		char *username = SvPV(ST(2), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	bgroup = group_find(priv, group);
	if (group == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(buddy_add(priv, target, bgroup, 1) == NULL);
}

XS(PORK_buddy_add_block) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(buddy_add_block(priv, target, 1));
}

XS(PORK_buddy_add_permit) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(buddy_add_permit(priv, target, 1));
}

XS(PORK_buddy_alias) {
	int notused;
	char *target;
	char *alias;
	struct buddy *buddy;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 2 && items != 3)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	alias = SvPV(ST(1), notused);

	if (target == NULL || alias == NULL)
		XSRETURN_IV(-1);

	if (items == 3) {
		char *username = SvPV(ST(2), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	buddy = buddy_find(priv, target);
	if (buddy == NULL)
		XSRETURN_IV(-1);

	buddy_alias(priv, buddy, alias);
	XSRETURN_IV(0);
}

XS(PORK_buddy_remove_block) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(buddy_remove_block(priv, target, 1));
}

XS(PORK_buddy_remove) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(buddy_remove(priv, target, 1));
}

XS(PORK_buddy_remove_permit) {
	int notused;
	char *target;
	struct aim_priv *priv;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	target = SvPV(ST(0), notused);
	if (target == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);
		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	XSRETURN_IV(buddy_remove_permit(priv, target, 1));
}

/*
** Scroll commands
*/

XS(PORK_scroll_by) {
	int lines;
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	lines = SvIV(ST(0));

	if (items == 2) {
		u_int32_t refnum = SvIV(ST(1));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_by(imwindow, lines);
	XSRETURN_IV(0);
}

XS(PORK_scroll_down) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_down(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_scroll_end) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_end(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_scroll_page_down) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_page_down(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_scroll_page_up) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_page_up(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_scroll_start) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_start(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_scroll_up) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_scroll_up(imwindow);
	XSRETURN_IV(0);
}

/*
** window commands
*/

XS(PORK_win_find_name) {
	char *str;
	int notused;
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	str = SvPV(ST(0), notused);
	if (str == NULL)
		XSRETURN_IV(-1);

	imwindow = imwindow_find_name(str);
	if (imwindow == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(imwindow->refnum);
}

XS(PORK_win_find_target) {
	char *str;
	int notused;
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	str = SvPV(ST(0), notused);
	if (str == NULL)
		XSRETURN_IV(-1);

	imwindow = imwindow_find(str);
	if (imwindow == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(imwindow->refnum);
}

XS(PORK_win_bind) {
	char *account;
	int notused;
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	account = SvPV(ST(0), notused);
	if (account == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		u_int32_t refnum = SvIV(ST(1));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	XSRETURN_IV(imwindow_bind_acct(imwindow, account));
}

XS(PORK_win_clear) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_clear(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_win_close) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	XSRETURN_IV(screen_close_window(imwindow));
}

XS(PORK_win_erase) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_erase(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_win_next) {
	dXSARGS;

	(void) cv;

	if (items != 0)
		XSRETURN_IV(-1);

	screen_cycle_fwd();
	XSRETURN_IV(0);
}

XS(PORK_win_prev) {
	dXSARGS;

	(void) cv;

	if (items != 0)
		XSRETURN_IV(-1);

	screen_cycle_bak();
	XSRETURN_IV(0);
}

XS(PORK_win_rename) {
	dXSARGS;
	struct imwindow *imwindow;
	char *new_name;
	int notused;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	new_name = SvPV(ST(0), notused);
	if (new_name == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		u_int32_t refnum = SvIV(ST(1));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_rename(imwindow, new_name);
	XSRETURN_IV(0);
}

XS(PORK_win_renumber) {
	dXSARGS;
	struct imwindow *imwindow;
	u_int32_t new_refnum;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	new_refnum = SvIV(ST(0));

	if (items == 2) {
		u_int32_t refnum = SvIV(ST(1));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	XSRETURN_IV(screen_renumber(imwindow, new_refnum));
}

XS(PORK_win_get_opt) {
	struct imwindow *imwindow;
	char *val;
	int unused;
	char buf[2048];

	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_UNDEF;

	val = SvPV(ST(0), unused);
	if (val == NULL)
		XSRETURN_UNDEF;

	if (items == 2) {
		u_int32_t refnum = SvIV(ST(1));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_UNDEF;
	} else
		imwindow = cur_window();

	if (wopt_get_val(imwindow, val, buf, sizeof(buf)) == -1)
		XSRETURN_UNDEF;

	XSRETURN_PV(buf);
}

XS(PORK_win_set_opt) {
	char *var;
	char *val;
	int unused;
	int opt;
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 2 && items != 3)
		XSRETURN_IV(-1);

	var = SvPV(ST(0), unused);
	val = SvPV(ST(1), unused);

	if (var == NULL || var == NULL)
		XSRETURN_IV(-1);

	opt = wopt_find(var);
	if (opt == -1)
		XSRETURN_IV(-1);

	if (items == 3) {
		u_int32_t refnum = SvIV(ST(2));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	XSRETURN_IV(wopt_set(imwindow, opt, val));
}

XS(PORK_win_swap) {
	u_int32_t refnum;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	refnum = SvIV(ST(0));
	XSRETURN_IV(screen_goto_window(refnum));
}

/*
** Timer commands.
*/

XS(PORK_timer_add) {
	char *command;
	int interval;
	int persistent;
	int unused;
	dXSARGS;

	(void) cv;

	if (items != 3)
		XSRETURN_IV(-1);

	interval = SvIV(ST(0));
	persistent = SvIV(ST(1));
	command = SvPV(ST(2), unused);

	if (command == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(timer_add(&screen.timer_list, command, interval, persistent));
}

XS(PORK_timer_del) {
	char *command;
	int unused;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	command = SvPV(ST(0), unused);
	if (command == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(timer_del(&screen.timer_list, command));
}

XS(PORK_timer_del_refnum) {
	u_int32_t refnum;
	dXSARGS;

	(void) cv;

	if (items != 1)
		XSRETURN_IV(-1);

	refnum = SvIV(ST(0));
	XSRETURN_IV(timer_del_refnum(&screen.timer_list, refnum));
}

XS(PORK_timer_purge) {
	dXSARGS;

	(void) items;
	(void) cv;

	timer_destroy(&screen.timer_list);
	XSRETURN_IV(0);
}

/*
** /blist commands.
*/

XS(PORK_blist_collapse) {
	struct aim_priv *priv;
	struct blist *blist;
	int notused;
	char *group;
	struct bgroup *bgroup;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	group = SvPV(ST(0), notused);
	if (group == NULL)
		XSRETURN_IV(-1);

	if (items == 2) {
		char *username = SvPV(ST(1), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	bgroup = group_find(priv, group);
	if (bgroup == NULL)
		XSRETURN_IV(-1);

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	blist_collapse_group(blist, bgroup);
	XSRETURN_IV(0);
}

XS(PORK_blist_cursor) {
	struct aim_priv *priv;
	struct blist *blist;
	struct slist_cell *cell;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_UNDEF;

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_UNDEF;

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_UNDEF;
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_UNDEF;

	cell = blist_get_cursor(blist);
	if (cell == NULL)
		XSRETURN_UNDEF;

	if (cell->type == TYPE_FLAT_CELL) {
		struct buddy *buddy = cell->data;

		XST_mPV(0, buddy->nname);
	} else {
		struct bgroup *group = cell->data;

		XST_mPV(0, group->name);
	}

	XST_mIV(1, cell->type);
	XSRETURN(2);
}

XS(PORK_blist_down) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_down(blist));
}

XS(PORK_blist_end) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_end(blist));
}

XS(PORK_blist_hide) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_blist_hide(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_blist_page_down) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_pgdown(blist));
}

XS(PORK_blist_page_up) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_pgup(blist));
}

XS(PORK_blist_refresh) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_refresh(blist));
}

XS(PORK_blist_show) {
	struct imwindow *imwindow;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		u_int32_t refnum = SvIV(ST(0));

		imwindow = imwindow_find_refnum(refnum);
		if (imwindow == NULL)
			XSRETURN_IV(-1);
	} else
		imwindow = cur_window();

	imwindow_blist_show(imwindow);
	XSRETURN_IV(0);
}

XS(PORK_blist_start) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_start(blist));
}

XS(PORK_blist_up) {
	struct aim_priv *priv;
	struct blist *blist;
	dXSARGS;

	(void) cv;

	if (items != 0 && items != 1)
		XSRETURN_IV(-1);

	if (items == 1) {
		int notused;
		char *username = SvPV(ST(0), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(blist_cursor_up(blist));
}

XS(PORK_blist_width) {
	struct aim_priv *priv;
	struct blist *blist;
	int new_width;
	dXSARGS;

	(void) cv;

	if (items != 1 && items != 2)
		XSRETURN_IV(-1);

	new_width = SvIV(ST(0));

	if (items == 2) {
		int notused;
		char *username = SvPV(ST(1), notused);

		if (username == NULL)
			XSRETURN_IV(-1);

		priv = pork_acct_get_priv(username);
		if (priv == NULL)
			XSRETURN_IV(-1);
	} else {
		struct imwindow *win = cur_window();
		priv = win->priv;
	}

	blist = priv->blist;
	if (blist == NULL)
		XSRETURN_IV(-1);

	XSRETURN_IV(screen_blist_width(blist, new_width));
}

void xs_init(init_args) {
	char *file = __FILE__;

	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);

	newXS("PORK::status_msg", PORK_status_msg, file);
	newXS("PORK::err_msg", PORK_err_msg, file);
	newXS("PORK::echo", PORK_echo, file);
	newXS("PORK::quit", PORK_quit, file);
	newXS("PORK::refresh", PORK_refresh, file);
	newXS("PORK::save", PORK_save, file);
	newXS("PORK::bind", PORK_bind, file);
	newXS("PORK::bind_get", PORK_bind_get, file);
	newXS("PORK::unbind", PORK_unbind, file);
	newXS("PORK::blist_bind", PORK_blist_bind, file);
	newXS("PORK::blist_bind_get", PORK_blist_bind_get, file);
	newXS("PORK::blist_unbind", PORK_blist_unbind, file);
	newXS("PORK::alias", PORK_alias, file);
	newXS("PORK::unalias", PORK_unalias, file);
	newXS("PORK::alias_get", PORK_alias_get, file);
	newXS("PORK::get_opt", PORK_get_opt, file);
	newXS("PORK::set_opt", PORK_set_opt, file);

	newXS("PORK::win_find_target", PORK_win_find_target, file);
	newXS("PORK::win_find_name", PORK_win_find_name, file);
	newXS("PORK::win_bind", PORK_win_bind, file);
	newXS("PORK::win_clear", PORK_win_clear, file);
	newXS("PORK::win_close", PORK_win_close, file);
	newXS("PORK::win_erase", PORK_win_erase, file);
	newXS("PORK::win_next", PORK_win_next, file);
	newXS("PORK::win_prev", PORK_win_prev, file);
	newXS("PORK::win_rename", PORK_win_rename, file);
	newXS("PORK::win_renumber", PORK_win_renumber, file);
	newXS("PORK::win_get_opt", PORK_win_get_opt, file);
	newXS("PORK::win_set_opt", PORK_win_set_opt, file);
	newXS("PORK::win_swap", PORK_win_swap, file);

	newXS("PORK::buddy_add", PORK_buddy_add, file);
	newXS("PORK::buddy_add_block", PORK_buddy_add_block, file);
	newXS("PORK::buddy_add_permit", PORK_buddy_add_permit, file);
	newXS("PORK::buddy_alias", PORK_buddy_alias, file);
	newXS("PORK::buddy_remove", PORK_buddy_remove, file);
	newXS("PORK::buddy_remove_block", PORK_buddy_remove_block, file);
	newXS("PORK::buddy_remove_permit", PORK_buddy_remove_permit, file);

	newXS("PORK::aim_permdeny_mode", PORK_aim_permdeny_mode, file);
	newXS("PORK::aim_get_buddy_away", PORK_aim_get_buddy_away, file);
	newXS("PORK::aim_get_buddy_profile", PORK_aim_get_buddy_profile, file);
	newXS("PORK::aim_set_profile", PORK_aim_set_profile, file);
	newXS("PORK::aim_get_profile", PORK_aim_get_profile, file);
	newXS("PORK::aim_send_im", PORK_aim_send_im, file);
	newXS("PORK::aim_report_idle", PORK_aim_report_idle, file);
	newXS("PORK::aim_search", PORK_aim_search, file);
	newXS("PORK::aim_set_away", PORK_aim_set_away, file);
	newXS("PORK::aim_set_idle", PORK_aim_set_idle, file);
	newXS("PORK::aim_warn", PORK_aim_warn, file);

	newXS("PORK::scroll_by", PORK_scroll_by, file);
	newXS("PORK::scroll_down", PORK_scroll_down, file);
	newXS("PORK::scroll_end", PORK_scroll_end, file);
	newXS("PORK::scroll_page_down", PORK_scroll_page_down, file);
	newXS("PORK::scroll_page_up", PORK_scroll_page_up, file);
	newXS("PORK::scroll_start", PORK_scroll_start, file);
	newXS("PORK::scroll_up", PORK_scroll_up, file);

	newXS("PORK::timer_add", PORK_timer_add, file);
	newXS("PORK::timer_del", PORK_timer_del, file);
	newXS("PORK::timer_del_refnum", PORK_timer_del_refnum, file);
	newXS("PORK::timer_purge", PORK_timer_purge, file);

	newXS("PORK::blist_collapse", PORK_blist_collapse, file);
	newXS("PORK::blist_cursor", PORK_blist_cursor, file);
	newXS("PORK::blist_down", PORK_blist_down, file);
	newXS("PORK::blist_end", PORK_blist_end, file);
	newXS("PORK::blist_hide", PORK_blist_hide, file);
	newXS("PORK::blist_page_down", PORK_blist_page_down, file);
	newXS("PORK::blist_page_up", PORK_blist_page_up, file);
	newXS("PORK::blist_refresh", PORK_blist_refresh, file);
	newXS("PORK::blist_show", PORK_blist_show, file);
	newXS("PORK::blist_start", PORK_blist_start, file);
	newXS("PORK::blist_up", PORK_blist_up, file);
	newXS("PORK::blist_width", PORK_blist_width, file);
}

int perl_load_file(char *filename) {
	char *atmp[] = { filename, NULL };

	return (execute_perl("load_n_eval", atmp));
}

int execute_perl(char *function, char **args) {
	int count;
	int ret_value = 1;
	SV *sv;

	dSP;
	ENTER;
	SAVETMPS;
	PUSHMARK(sp);

	count = perl_call_argv(function, G_EVAL | G_SCALAR, args);
	SPAGAIN;

	sv = GvSV(gv_fetchpv("@", TRUE, SVt_PV));
	if (SvTRUE(sv)) {
		screen_status_msg("Perl error: %s", SvPV(sv, count));
		(void) POPs;
	} else if (count != 1) {
		screen_status_msg("Perl error: expected 1 value from %s, got %d",
			function, count);
	} else {
		ret_value = POPi;
	}

	PUTBACK;
	FREETMPS;
	LEAVE;

	return (ret_value);
}

int perl_init(void) {
	static char *perl_args[] = { "", "-e", "0", "-w" };
	static char perl_definitions[] = {
		"sub load_file{"
			"my $f_name=shift;"
			"local $/=undef;"
			"open FH,$f_name or return \"__FAILED__\";"
			"$_=<FH>;"
			"close FH;"
			"return $_;"
		"}"
		"sub load_n_eval{"
			"my $f_name=shift;"
			"my $strin=load_file($f_name);"
			"return 2 if($strin eq \"__FAILED__\");"
			"eval $strin;"
			"if($@){"
				/*" #something went wrong\n"*/
				"PORK::screen_status_msg(\"While loading file '$f_name': $@\");"
				"return 1;"
			"}"
			"return 0;"
		"}"
	};

	perl_inter = perl_alloc();
	if (perl_inter == NULL)
		return (-1);

	perl_construct(perl_inter);
	perl_parse(perl_inter, xs_init, 3, perl_args, NULL);

#ifdef HAVE_PERL_EVAL_PV
	eval_pv(perl_definitions, TRUE);
#else
	perl_eval_pv(perl_definitions, TRUE);
#endif

	return (0);
}

int perl_destroy(void) {
	if (perl_inter != NULL) {
		perl_destruct(perl_inter);
		perl_free(perl_inter);

		return (0);
	}

	return (-1);
}

#else

#include <pork_util.h>
#include <pork_perl.h>

/*
** Stubs for when Perl support isn't enabled.
*/

int perl_init(void) {
	return (0);
}

int perl_destroy(void) {
	return (0);
}

int execute_perl(char *function __notused, char **args __notused) {
	return (0);
}

int perl_load_file(char *filename __notused) {
	return (-1);
}

#endif
